@@ -5,9 +5,9 @@
 # from a C++ header
 #
 # Author: Matt Morse (matt@apple.com)
-# Last Updated: $Date: 2001/03/22 02:27:13 $
+# Last Updated: $Date: 2003/05/31 01:43:26 $
 # 
-# Copyright (c) 1999 Apple Computer, Inc.  All Rights Reserved.
+# Copyright (c) 1999-2001 Apple Computer, Inc.  All Rights Reserved.
 # The contents of this file constitute Original Code as defined in and are
 # subject to the Apple Public Source License Version 1.1 (the "License").
 # You may not use this file except in compliance with the License.  Please
@@ -30,14 +30,15 @@ BEGIN {
 }
 package HeaderDoc::CPPClass;
 
-use HeaderDoc::Utilities qw(findRelativePath safeName getAPINameAndDisc convertCharsForFileMaker printArray printHash);
+use HeaderDoc::Utilities qw(findRelativePath safeName getAPINameAndDisc printArray printHash);
 use HeaderDoc::APIOwner;
-@ISA = qw( HeaderDoc::APIOwner );
 
 use strict;
 use vars qw($VERSION @ISA);
 $VERSION = '1.20';
 
+# Inheritance
+@ISA = qw( HeaderDoc::APIOwner );
 ################ Portability ###################################
 my $isMacOS;
 my $pathSeparator;
@@ -61,62 +62,22 @@ $year += 1900;
 my $dateStamp = "$moy/$dom/$year";
 ######################################################################
 
-sub new {
-    my($param) = shift;
-    my($class) = ref($param) || $param;
-    my $self = {};
-    
-    bless($self, $class);
-    $self->SUPER::_initialize();
-    $self->_initialize();
-    print "Created new CPPClass object\n" if ($debugging);
-    return($self);
-}
-
 sub _initialize {
     my($self) = shift;
-    $self->{HEADEROBJECT} = undef;
-}
-
-sub headerObject {
-    my $self = shift;
-
-    if (@_) {
-        $self->{HEADEROBJECT} = shift;
-    }
-    return $self->{HEADEROBJECT};
-}
-
-sub createTOCFile {
-    my $self = shift;
-    my $rootDir = $self->outputDir();
-    my $outputFileName = "toc.html";    
-    my $outputFile = "$rootDir$pathSeparator$outputFileName";    
-    my $fileString = $self->tocString();    
-    my $filename = $self->name();    
-
-	open(OUTFILE, ">$outputFile") || die "Can't write $outputFile.\n$!\n";
-    if ($isMacOS) {MacPerl::SetFileInfo('MSIE', 'TEXT', "$outputFile");};
-	print OUTFILE "<html><head><title>Draft Documentation for $filename</title></head>\n";
-	print OUTFILE "<body bgcolor=\"#cccccc\">\n";
-	print OUTFILE "<table border=\"0\" cellpadding=\"0\" cellspacing=\"2\" width=\"148\">\n";
-	print OUTFILE "<tr><td colspan=\"2\"><font size=\"5\" color=\"#330066\"><b>Class:</b></font></td></tr>\n";
-	print OUTFILE "<tr><td width=\"15\"></td><td><b><font size=\"+1\">$filename</font></b></td></tr>\n";
-	print OUTFILE "</table><hr>\n";
-	print OUTFILE $fileString;
-	print OUTFILE "</body></html>\n";
-	close OUTFILE;
+    $self->SUPER::_initialize();
+    $self->tocTitlePrefix('Class:');
 }
 
 sub tocString {
     my $self = shift;
+    my $localDebug = 0;
     my $contentFrameName = $self->name();
     $contentFrameName =~ s/(.*)\.h/$1/; 
     # for now, always shorten long names since some files may be moved to a Mac for browsing
-    if (1 || $isMacOS) {$contentFrameName = &safeName($contentFrameName);};
+    if (1 || $isMacOS) {$contentFrameName = &safeName(filename => $contentFrameName);};
     $contentFrameName = $contentFrameName . ".html";
     my $header = $self->headerObject();
-    my @funcs = $self->functions();
+    my @funcs = $self->functions();    
     my @constants = $self->constants();
     my @typedefs = $self->typedefs();
     my @structs = $self->structs();
@@ -125,14 +86,27 @@ sub tocString {
 	my $compositePageName = HeaderDoc::APIOwner->compositePageName();
 	my $defaultFrameName = HeaderDoc::APIOwner->defaultFrameName();
     
-    my $tocString = "<h3><a href=\"$contentFrameName\" target =\"doc\">Introduction</a></h3>\n";
+    my $tocString;
+
+    if ($self->outputformat eq "hdxml") {
+ 	$tocString = "XMLFIX<h3><a href=\"$contentFrameName\" target=\"doc\">Introduction</a></h3>\n";
+    } elsif ($self->outputformat eq "html") {
+ 	$tocString = "<nobr>&nbsp;<a href=\"$contentFrameName\" target=\"doc\">Introduction</a>\n";
+    } else {
+	$tocString = "UNKNOWN OUTPUT FORMAT TYPE";
+    }
 
     # output list of functions as TOC
     if (@funcs) {
         my @publics;
         my @protecteds;
         my @privates;
-	    $tocString .= "<br><h4>Member Functions</h4><hr>\n";
+	    if ($self->outputformat eq "hdxml") {
+	        $tocString .= "XMLFIX<br><h4>Member Functions</h4><hr>\n";
+	    } elsif ($self->outputformat eq "html") {
+	        $tocString .= "<br><h4>Member Functions</h4><hr>\n";
+            } else {
+	    }
 	    foreach my $obj (sort byAccessControl @funcs) {
 	        my $access = $obj->accessControl();
 	        
@@ -145,53 +119,113 @@ sub tocString {
 	        }
 	    }
 	    if (@publics) {
-	        $tocString .= "<h5>Public</h5>\n";
+	        if ($self->outputformat eq "hdxml") {
+	            $tocString .= "XMLFIX<h5>Public</h5>\n";
+	        } elsif ($self->outputformat eq "html") {
+	            $tocString .= "<h5>Public</h5>\n";
+		} else {
+		}
 		    foreach my $obj (sort objName @publics) {
 	        	my $name = $obj->name();
-	        	$tocString .= "<nobr>&nbsp;<a href = \"Functions/Functions.html#$name\" target =\"doc\">$name</a></nobr><br>\n";
+	        	if ($self->outputformat eq "hdxml") {
+	        	    $tocString .= "XMLFIX<nobr>&nbsp;<a href=\"Functions/Functions.html#$name\" target=\"doc\">$name</a></nobr><br>\n";
+		        } elsif ($self->outputformat eq "html") {
+	        	    $tocString .= "<nobr>&nbsp;<a href=\"Functions/Functions.html#$name\" target=\"doc\">$name</a></nobr><br>\n";
+			} else {
+			}
 	        }
 	    }
 	    if (@protecteds) {
-	        $tocString .= "<h5>Protected</h5>\n";
+	        if ($self->outputformat eq "hdxml") {
+	            $tocString .= "XMLFIX<h5>Protected</h5>\n";
+	        } elsif ($self->outputformat eq "html") {
+	            $tocString .= "<h5>Protected</h5>\n";
+		} else {
+		}
 		    foreach my $obj (sort objName @protecteds) {
 	        	my $name = $obj->name();
-	        	$tocString .= "<nobr>&nbsp;<a href = \"Functions/Functions.html#$name\" target =\"doc\">$name</a></nobr><br>\n";
+		        if ($self->outputformat eq "hdxml") {
+	        	    $tocString .= "XMLFIX<nobr>&nbsp;<a href=\"Functions/Functions.html#$name\" target=\"doc\">$name</a></nobr><br>\n";
+		        } elsif ($self->outputformat eq "html") {
+	        	    $tocString .= "<nobr>&nbsp;<a href=\"Functions/Functions.html#$name\" target=\"doc\">$name</a></nobr><br>\n";
+			} else {
+			}
 	        }
 	    }
 	    if (@privates) {
-	        $tocString .= "<h5>Private</h5>\n";
+	        if ($self->outputformat eq "hdxml") {
+	            $tocString .= "XMLFIX<h5>Private</h5>\n";
+	        } elsif ($self->outputformat eq "html") {
+	            $tocString .= "<h5>Private</h5>\n";
+		} else {
+		}
 		    foreach my $obj (sort objName @privates) {
 	        	my $name = $obj->name();
-	        	$tocString .= "<nobr>&nbsp;<a href = \"Functions/Functions.html#$name\" target =\"doc\">$name</a></nobr><br>\n";
+	        	if ($self->outputformat eq "hdxml") {
+	        	    $tocString .= "XMLFIX<nobr>&nbsp;<a href=\"Functions/Functions.html#$name\" target=\"doc\">$name</a></nobr><br>\n";
+		        } elsif ($self->outputformat eq "html") {
+	        	    $tocString .= "<nobr>&nbsp;<a href=\"Functions/Functions.html#$name\" target=\"doc\">$name</a></nobr><br>\n";
+			} else {
+			}
 	        }
 	    }
     }
     if (@typedefs) {
-	    $tocString .= "<h4>Defined Types</h4>\n";
+       	    if ($self->outputformat eq "hdxml") {
+	        $tocString .= "XMLFIX<h4>Defined Types</h4>\n";
+	    } elsif ($self->outputformat eq "html") {
+	        $tocString .= "<h4>Defined Types</h4>\n";
+	    } else {
+	    }
 	    foreach my $obj (sort objName @typedefs) {
 	        my $name = $obj->name();
-	        $tocString .= "<nobr>&nbsp;<a href = \"DataTypes/DataTypes.html#$name\" target =\"doc\">$name</a></nobr><br>\n";
+       	        if ($self->outputformat eq "hdxml") {
+	            $tocString .= "XMLFIX<nobr>&nbsp;<a href=\"DataTypes/DataTypes.html#$name\" target=\"doc\">$name</a></nobr><br>\n";
+	        } elsif ($self->outputformat eq "html") {
+	            $tocString .= "<nobr>&nbsp;<a href=\"DataTypes/DataTypes.html#$name\" target=\"doc\">$name</a></nobr><br>\n";
+		} else {
+		}
 	    }
     }
     if (@structs) {
 	    $tocString .= "<h4>Structs</h4>\n";
 	    foreach my $obj (sort objName @structs) {
 	        my $name = $obj->name();
-	        $tocString .= "<nobr>&nbsp;<a href = \"Structs/Structs.html#$name\" target =\"doc\">$name</a></nobr><br>\n";
+       	        if ($self->outputformat eq "hdxml") {
+	            $tocString .= "XMLFIX<nobr>&nbsp;<a href=\"Structs/Structs.html#$name\" target=\"doc\">$name</a></nobr><br>\n";
+	        } elsif ($self->outputformat eq "html") {
+	            $tocString .= "<nobr>&nbsp;<a href=\"Structs/Structs.html#$name\" target=\"doc\">$name</a></nobr><br>\n";
+		} else {
+		}
 	    }
     }
     if (@constants) {
 	    $tocString .= "<h4>Constants</h4>\n";
 	    foreach my $obj (sort objName @constants) {
 	        my $name = $obj->name();
-	        $tocString .= "<nobr>&nbsp;<a href = \"Constants/Constants.html#$name\" target =\"doc\">$name</a></nobr><br>\n";
+       	        if ($self->outputformat eq "hdxml") {
+	            $tocString .= "XMLFIX<nobr>&nbsp;<a href=\"Constants/Constants.html#$name\" target=\"doc\">$name</a></nobr><br>\n";
+	        } elsif ($self->outputformat eq "html") {
+	            $tocString .= "<nobr>&nbsp;<a href=\"Constants/Constants.html#$name\" target=\"doc\">$name</a></nobr><br>\n";
+		} else {
+		}
 	    }
 	}
     if (@enums) {
-	    $tocString .= "<h4>Enumerations</h4>\n";
+       	    if ($self->outputformat eq "hdxml") {
+	        $tocString .= "XMLFIX<h4>Enumerations</h4>\n";
+	    } elsif ($self->outputformat eq "html") {
+	        $tocString .= "<h4>Enumerations</h4>\n";
+	    } else {
+	    }
 	    foreach my $obj (sort objName @enums) {
 	        my $name = $obj->name();
-	        $tocString .= "<nobr>&nbsp;<a href = \"Enums/Enums.html#$name\" target =\"doc\">$name</a></nobr><br>\n";
+       	        if ($self->outputformat eq "hdxml") {
+	            $tocString .= "XMLFIX<nobr>&nbsp;<a href=\"Enums/Enums.html#$name\" target=\"doc\">$name</a></nobr><br>\n";
+	        } elsif ($self->outputformat eq "html") {
+	            $tocString .= "<nobr>&nbsp;<a href=\"Enums/Enums.html#$name\" target=\"doc\">$name</a></nobr><br>\n";
+		} else {
+		}
 	    }
 	}
     if (@vars) {
@@ -199,7 +233,12 @@ sub tocString {
         my @protecteds;
         my @privates;
 
-	    $tocString .= "<br><h4>Member Data</h4><hr>\n";
+       	    if ($self->outputformat eq "hdxml") {
+	        $tocString .= "XMLFIX<br><h4>Member Data</h4><hr>\n";
+	    } elsif ($self->outputformat eq "html") {
+	        $tocString .= "<br><h4>Member Data</h4><hr>\n";
+	    } else {
+	    }
 	    foreach my $obj (sort byAccessControl @vars) {
 	        my $access = $obj->accessControl();
 
@@ -215,27 +254,30 @@ sub tocString {
 	        $tocString .= "<h5>Public</h5>\n";
 		    foreach my $obj (sort objName @publics) {
 	        	my $name = $obj->name();
-	        	$tocString .= "<nobr>&nbsp;<a href = \"Vars/Vars.html#$name\" target =\"doc\">$name</a></nobr><br>\n";
+	        	$tocString .= "<nobr>&nbsp;<a href=\"Vars/Vars.html#$name\" target=\"doc\">$name</a></nobr><br>\n";
 	        }
 	    }
 	    if (@protecteds) {
 	        $tocString .= "<h5>Protected</h5>\n";
 		    foreach my $obj (sort objName @protecteds) {
 	        	my $name = $obj->name();
-	        	$tocString .= "<nobr>&nbsp;<a href = \"Vars/Vars.html#$name\" target =\"doc\">$name</a></nobr><br>\n";
+	        	$tocString .= "<nobr>&nbsp;<a href=\"Vars/Vars.html#$name\" target=\"doc\">$name</a></nobr><br>\n";
 	        }
 	    }
 	    if (@privates) {
 	        $tocString .= "<h5>Private</h5>\n";
 		    foreach my $obj (sort objName @privates) {
 	        	my $name = $obj->name();
-	        	$tocString .= "<nobr>&nbsp;<a href = \"Vars/Vars.html#$name\" target =\"doc\">$name</a></nobr><br>\n";
+	        	$tocString .= "<nobr>&nbsp;<a href=\"Vars/Vars.html#$name\" target=\"doc\">$name</a></nobr><br>\n";
 	        }
 	    }
 	}
 	$tocString .= "<br><h4>Other Reference</h4><hr>\n";
-	$tocString .= "<nobr>&nbsp;<a href = \"../../$defaultFrameName\" target =\"_top\">Header</a></nobr><br>\n";
-    $tocString .= "<br><hr><a href=\"$compositePageName\" target =\"_blank\">[Printable HTML Page]</a>\n";
+	$tocString .= "<nobr>&nbsp;<a href=\"../../$defaultFrameName\" target=\"_top\">Header</a></nobr><br>\n";
+    $tocString .= "<br><hr><a href=\"$compositePageName\" target=\"_blank\">[Printable HTML Page]</a>\n";
+    
+	print "*** finished toc\n" if ($localDebug);
+	
     return $tocString;
 }
 
@@ -267,42 +309,49 @@ sub _getCompositePageString {
     $contentString= $self->_getFunctionDetailString();
     if (length($contentString)) {
 	    $compositePageString .= "<h2>Member Functions</h2>\n";
+		$contentString = $self->stripAppleRefs($contentString);
 	    $compositePageString .= $contentString;
     }
     
     $contentString= $self->_getVarDetailString();
     if (length($contentString)) {
 	    $compositePageString .= "<h2>Member Data</h2>\n";
+		$contentString = $self->stripAppleRefs($contentString);
 	    $compositePageString .= $contentString;
     }
     
     $contentString= $self->_getConstantDetailString();
     if (length($contentString)) {
 	    $compositePageString .= "<h2>Constants</h2>\n";
+		$contentString = $self->stripAppleRefs($contentString);
 	    $compositePageString .= $contentString;
     }
     
     $contentString= $self->_getTypedefDetailString();
     if (length($contentString)) {
 	    $compositePageString .= "<h2>Typedefs</h2>\n";
+		$contentString = $self->stripAppleRefs($contentString);
 	    $compositePageString .= $contentString;
     }
     
     $contentString= $self->_getStructDetailString();
     if (length($contentString)) {
 	    $compositePageString .= "<h2>Structs</h2>\n";
+		$contentString = $self->stripAppleRefs($contentString);
 	    $compositePageString .= $contentString;
     }
     
     $contentString= $self->_getEnumDetailString();
     if (length($contentString)) {
 	    $compositePageString .= "<h2>Enumerations</h2>\n";
+		$contentString = $self->stripAppleRefs($contentString);
 	    $compositePageString .= $contentString;
     }
 
     $contentString= $self->_getPDefineDetailString();
     if (length($contentString)) {
 	    $compositePageString .= "<h2>#defines</h2>\n";
+		$contentString = $self->stripAppleRefs($contentString);
 	    $compositePageString .= $contentString;
     }  
     return $compositePageString;
@@ -321,23 +370,39 @@ sub _getFunctionDetailString {
     foreach my $obj (sort objName @funcObjs) {
         my $name = $obj->name();
         my $desc = $obj->discussion();
+	my $throws = $obj->throws();
         my $abstract = $obj->abstract();
+        my $updated = $obj->updated();
         my $declaration = $obj->declarationInHTML();
         my $declarationRaw = $obj->declaration();
         my $accessControl = $obj->accessControl();
         my @params = $obj->taggedParameters();
         my $result = $obj->result();
-		if ($declaration !~ /#define/) { # not sure how to handle apple_refs with macros yet
+	$contentString .= "<hr>";
+	# if ($declaration !~ /#define/) { # not sure how to handle apple_refs with macros yet
 	        my $paramSignature = $self->getParamSignature($declarationRaw);
 	        my $methodType = $self->getMethodType($declarationRaw);
         	$contentString .= "<a name=\"//$apiUIDPrefix/cpp/$methodType/$className/$name/$paramSignature\"></a>\n";
+        # }
+	$contentString .= "<table border=\"0\"  cellpadding=\"2\" cellspacing=\"2\" width=\"300\">";
+	$contentString .= "<tr>";
+	$contentString .= "<td valign=\"top\" height=\"12\" colspan=\"5\">";
+	$contentString .= "<h2><a name=\"$name\">$name</a></h2>\n";
+	$contentString .= "</td>";
+	$contentString .= "</tr></table>";
+	$contentString .= "<hr>";
+
+	if (length($throws)) {  
+		$contentString .= "<b>Throws:</b>\n$throws<BR>\n";
+	}
+	if (length($abstract)) {
+            # $contentString .= "<b>Abstract:</b> $abstract<BR>\n";
+            $contentString .= "$abstract<BR>\n";
         }
-        $contentString .= "<h3><a name=\"$name\">$name</a></h3>\n";
-	    if (length($abstract)) {
-            $contentString .= "<b>Abstract:</b> $abstract\n";
-        }
+	if (length($updated)) {
+	    $contentString .= "<b>Updated:</b> $updated<BR>\n";
+	}
         $contentString .= "<blockquote><pre><tt>$accessControl</tt>\n<br>$declaration</pre></blockquote>\n";
-        $contentString .= "<p>$desc</p>\n";
 	    my $arrayLength = @params;
 	    if ($arrayLength > 0) {
 	        my $paramContentString;
@@ -345,26 +410,115 @@ sub _getFunctionDetailString {
 	            my $pName = $element->name();
 	            my $pDesc = $element->discussion();
 	            if (length ($pName)) {
-	                $paramContentString .= "<tr><td align = \"center\"><tt>$pName</tt></td><td>$pDesc</td><tr>\n";
+	                $paramContentString .= "<tr><td align=\"center\"><tt>$pName</tt></td><td>$pDesc</td></tr>\n";
 	            }
 	        }
 	        if (length ($paramContentString)){
 		        $contentString .= "<h4>Parameters</h4>\n";
 		        $contentString .= "<blockquote>\n";
-		        $contentString .= "<table border = \"1\"  width = \"90%\">\n";
+		        $contentString .= "<table border=\"1\"  width=\"90%\">\n";
 		        $contentString .= "<thead><tr><th>Name</th><th>Description</th></tr></thead>\n";
 	            $contentString .= $paramContentString;
 		        $contentString .= "</table>\n</blockquote>\n";
 		    }
 	    }
-	    if (length($result)) {
+	if (length($result)) {
             $contentString .= "<b>Result:</b> $result\n";
         }
-	    $contentString .= "<hr>\n";
+	if (length($desc)) {$contentString .= "<p>$desc</p>\n"; }
+	    # $contentString .= "<hr>\n";
     }
     return $contentString;
 }
 
+sub XMLdocumentationBlock {
+    my $self = shift;
+    my $compositePageString = "";
+    my $name = $self->name();
+    my $updated = $self->updated();
+    my $abstract = $self->abstract();
+    my $discussion = $self->discussion();
+    my $contentString;
+    
+    $compositePageString .= "<class type=\"C++\">";
+
+    if (length($name)) {
+	$compositePageString .= "<name>$name</name>\n";
+    }
+
+    if (length($abstract)) {
+	$compositePageString .= "<abstract>$abstract</abstract>\n";
+    }
+    if (length($updated)) {
+	$contentString .= "<updated>$updated</updated>\n";
+    }
+	my @fields = $self->fields();
+	if (@fields) {
+		$contentString .= "<template_fields>\n";
+		for my $field (@fields) {
+			my $name = $field->name();
+			my $desc = $field->discussion();
+			# print "field $name $desc\n";
+			$contentString .= "<field><name>$name</name><desc>$desc</desc></field>\n";
+		}
+		$contentString .= "</template_fields>\n";
+	}
+    if (length($discussion)) {
+	$compositePageString .= "<discussion>$discussion</discussion>\n";
+    }
+
+    $contentString= $self->_getFunctionXMLDetailString();
+    if (length($contentString)) {
+	$contentString = $self->stripAppleRefs($contentString);
+	$compositePageString .= "<functions>$contentString</functions>\n";
+    }
+
+    $contentString= $self->_getMethodXMLDetailString();
+    if (length($contentString)) {
+	$contentString = $self->stripAppleRefs($contentString);
+	$compositePageString .= "<methods>$contentString</methods>\n";
+    }
+    
+    $contentString= $self->_getVarXMLDetailString();
+    if (length($contentString)) {
+	$contentString = $self->stripAppleRefs($contentString);
+	$compositePageString .= "<globals>$contentString</globals>\n";
+    }
+    
+    $contentString= $self->_getConstantXMLDetailString();
+    if (length($contentString)) {
+	$contentString = $self->stripAppleRefs($contentString);
+	$compositePageString .= "<constants>$contentString</constants>\n";
+    }
+    
+    $contentString= $self->_getTypedefXMLDetailString();
+    if (length($contentString)) {
+	$contentString = $self->stripAppleRefs($contentString);
+	$compositePageString .= "<typedefs>$contentString</typedefs>";
+    }
+    
+    $contentString= $self->_getStructXMLDetailString();
+    if (length($contentString)) {
+	$contentString = $self->stripAppleRefs($contentString);
+	$compositePageString .= "<structs>$contentString</structs>";
+    }
+    
+    $contentString= $self->_getEnumXMLDetailString();
+    if (length($contentString)) {
+	$contentString = $self->stripAppleRefs($contentString);
+	$compositePageString .= "<enums>$contentString</enums>";
+    }
+
+    $contentString= $self->_getPDefineXMLDetailString();
+    if (length($contentString)) {
+	$contentString = $self->stripAppleRefs($contentString);
+	$compositePageString .= "<defines>$contentString</defines>";
+    }  
+
+    $compositePageString .= "</class>";
+    return $compositePageString;
+}
+
 sub _getVarDetailString {
     my $self = shift;
     my @varObjs = $self->vars();
@@ -372,6 +526,8 @@ sub _getVarDetailString {
 
     foreach my $obj (sort objName @varObjs) {
         my $name = $obj->name();
+	my $abstract = $obj->abstract();
+	my $updated = $obj->updated();
         my $desc = $obj->discussion();
         my $declaration = $obj->declarationInHTML();
         my $accessControl = $obj->accessControl();
@@ -389,28 +545,62 @@ sub _getVarDetailString {
             	$fieldHeading = "Parameters";
         	}
         }
+
+	my $methodType = "var"; # $self->getMethodType($declarationRaw);
+	my $apiUIDPrefix = HeaderDoc::APIOwner->apiUIDPrefix();
+	my $headerObject = HeaderDoc::APIOwner->headerObject();
+	my $className = (HeaderDoc::APIOwner->headerObject())->name();
+	$contentString .= "<hr>";
+	$contentString .= "<a name=\"//$apiUIDPrefix/occ/$methodType/$className/$name\"></a>\n";
         
-        $contentString .= "<h3><a name=\"$name\">$name</a></h3>\n";
-        $contentString .= "<blockquote><tt>$accessControl</tt>$declaration</blockquote>\n";
+	$contentString .= "<table border=\"0\"  cellpadding=\"2\" cellspacing=\"2\" width=\"300\">";
+	$contentString .= "<tr>";
+	$contentString .= "<td valign=\"top\" height=\"12\" colspan=\"5\">";
+	$contentString .= "<h2><a name=\"$name\">$name</a></h2>\n";
+	$contentString .= "</td>";
+	$contentString .= "</tr></table>";
+	$contentString .= "<hr>";
+	if (length($abstract)) {
+		# $contentString .= "<b>Abstract:</b> $abstract<BR>\n";
+		$contentString .= "$abstract<BR>\n";
+	}
+	if (length($updated)) {
+		$contentString .= "<b>Updated:</b> $updated<BR>\n";
+	}
+        $contentString .= "<blockquote><tt>$accessControl</tt> $declaration</blockquote>\n";
         $contentString .= "<p>$desc</p>\n";
 	    my $arrayLength = @fields;
 	    if ($arrayLength > 0) {
 	        $contentString .= "<h4>$fieldHeading</h4>\n";
 	        $contentString .= "<blockquote>\n";
-	        $contentString .= "<table border = \"1\"  width = \"90%\">\n";
+	        $contentString .= "<table border=\"1\"  width=\"90%\">\n";
 	        $contentString .= "<thead><tr><th>Name</th><th>Description</th></tr></thead>\n";
 	        foreach my $element (@fields) {
 	            my $fName = $element->name();
 	            my $fDesc = $element->discussion();
-	            $contentString .= "<tr><td align = \"center\"><tt>$fName</tt></td><td>$fDesc</td><tr>\n";
+	            $contentString .= "<tr><td align=\"center\"><tt>$fName</tt></td><td>$fDesc</td></tr>\n";
 	        }
 	        $contentString .= "</table>\n</blockquote>\n";
 	    }
+	    if (length($updated)) {
+		$contentString .= "<b>Updated:</b> $updated\n";
+	    }
 	    $contentString .= "<hr>\n";
     }
     return $contentString;
 }
 
+sub getMethodType {
+    my $self = shift;
+	my $declaration = shift;
+	my $methodType = "instm";
+	
+	if ($declaration =~ /^\s*static/) {
+	    $methodType = "clm";
+	}
+	return $methodType;
+}
+
 sub getParamSignature {
     my $self = shift;
 	my $declaration = shift;
@@ -428,30 +618,18 @@ sub getParamSignature {
 	return $sig;
 }
 
-sub getMethodType {
-    my $self = shift;
-	my $declaration = shift;
-	my $methodType = "instm";
-	
-	if ($declaration =~ /^\s*static/) {
-	    $methodType = "clm";
-	}
-	return $methodType;
-}
-
-
+# we add the apple_ref markup to the navigator comment to identify
+# to Project Builder and other applications indexing the documentation
+# that this is the entry point for documentation for this class
 sub docNavigatorComment {
     my $self = shift;
     my $name = $self->name();
+    my $navComment = "<!-- headerDoc=cl; name=$name-->";
+    my $appleRef = "<a name=\"//apple_ref/cpp/cl/$name\"></a>";
     
-    return "<-- headerDoc=CPPClass; name=$name-->";
+    return "$navComment\n$appleRef";
 }
 
-
-
-
-
-
 ################## Misc Functions ###################################
 
 
@@ -486,9 +664,8 @@ sub linkageAndObjName { # used for sorting
 sub printObject {
     my $self = shift;
  
-    print "CPPClass\n";
     $self->SUPER::printObject();
-    print "headerObject: $self->{HEADEROBJECT}\n";
+    print "CPPClass\n";
     print "\n";
 }
 
